// -*- C++ -*-
/*!
 * @file  art.cpp
 * @brief ARToolkit image capture component
 * @date $Date$
 *
 * $Id$
 */

#include "art.h"
#include <iostream>
using namespace std;

// Module specification
// <rtc-template block="module_spec">
static const char* art_spec[] =
  {
    "implementation_id", "art",
    "type_name",         "ARToolKit-RTC",
    "description",       "Augmented Reality RT-Component",
    "version",           "1.0.0",
    "vendor",            "JANG MINHO, LEE HYUNDEOK",
    "category",          "example",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "100.0",

		// Configuration variables
		"conf.default.FILE Camera_Parameter", "camera/camera_para.dat",
    "conf.default.FILE Marker", "marker/Char_OpenRTM-aist",
		"conf.default.FILE VRML", "vrml/HRP4C",
		"conf.default.Marker_Check Monochrome(0-255)", "100",
		
		"conf.default.3D_Translate X", "0",
		"conf.default.3D_Translate Y", "0",
		"conf.default.3D_Translate Z", "0",
		"conf.default.3D_Rotate ANGLE", "0",
		"conf.default.3D_Rotate X", "0",
		"conf.default.3D_Rotate Y", "0",
		"conf.default.3D_Rotate Z", "0",
		"conf.default.3D_Scale ALL", "1.0",
		"conf.default.3D_Scale X", "1.0",
		"conf.default.3D_Scale Y", "1.0",
		"conf.default.3D_Scale Z", "1.0",

		"conf.default.HRP3_RLEG Front Back", "0",
		""
  };
// </rtc-template>

// ============================================================================
//	Global variables
// ============================================================================
// O[oϐ

std::string	cparam_name;		// Jp[^t@C
std::string	patt_name;			// p^[t@C
std::string	vrml_name;			// VRML̐ݒt@C
int		thresh;								// 2l臒l

// VRML Model control 
GLfloat translate_x;
GLfloat translate_y;
GLfloat translate_z;
GLfloat rotate_angle;
GLfloat rotate_x;
GLfloat rotate_y;
GLfloat rotate_z;
GLfloat scale_all;
GLfloat scale_x_all;
GLfloat scale_y_all;
GLfloat scale_z_all;
GLfloat scale_x;
GLfloat scale_y;
GLfloat scale_z;

GLfloat rotate_angle_RLEG;

int		patt_id;						// p^[ID
int		vrml_id;						// VRMLID
int		vrml_id_hrp[10];

ARParam	cparam;			// Jp[^
ARParam	wparam;			// Jp[^iƗpϐj

double	patt_trans[3][4];				// Wϊs
double	patt_center[2]	= { 0.0, 0.0 };	// p^[̒SW
double	patt_width		= 80.0;			// p^[̃TCYiPʁFmmj

// ̓C[WTemp
int m_in_height = 0;					// ̓C[WTempHeight
int m_in_width = 0;						// ̓C[WTempWidth

// flag
bool ARMainLoopStopFlag = FALSE;
bool paramInitFlag = FALSE;
bool scale_all_flag = FALSE;

// Frame Buffer Objext var
GLuint texture_name;
GLuint renderbuffer_name;
GLuint framebuffer_name;

// image global var
ARUint8*	ar_image;
IplImage* opencv_img_send_tmp;
IplImage* opencv_img_send_mono_tmp;

//////////////// C[Wϐ̕ϊ菇 ////////////////////////
//
//---- onExecute() ----
// m_in.pixels[0] 
// -> opencv_image(cv`p) 
// -> opencv_marker_image(cvop) -> ar_image(arop)

//---- AR_MainLoop() ----
// ar_image(arop)
//			1)	ar_image(2D camera image) -> buffer 
//			2)	ar_image(2D camera image) -> p^[F -> 3D object -> buffer
// buffer
// -> opencv_img_send_tmp(cvop) 

//---- onExecute() ----
// opencv_img_send_tmp(cvop) 
// -> opencv_img_send(cv`p) 
// -> m_out.pixels[0]
/////////////////////////////////////////////////////////////////

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
art::art(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
		m_inIn("Camera_Image_IN", m_in),
		m_outOut("AR_Rendering_Image_OUT", m_out),
		m_out_monoOut("Marker_Detect_Mono_OUT", m_out_mono),
    dummy(0)

    // </rtc-template>
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("Camera_Image_IN", m_inIn);
  registerOutPort("AR_Rendering_Image_OUT", m_outOut);
	registerOutPort("Marker_Detect_Mono_OUT", m_out_monoOut);

	// Set OutPort buffer
   
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

/*!
 * @brief destructor
 */
art::~art()
{
}



RTC::ReturnCode_t art::onInitialize()
{
	bindParameter("FILE Camera_Parameter", cfg_cparam_name, "camera/camera_para.dat");
  bindParameter("FILE Marker", cfg_patt_name,   "marker/Char_OpenRTM-aist");
  bindParameter("FILE VRML", cfg_vrml_name,   "vrml/HRP4C");
  bindParameter("Marker_Check Monochrome(0-255)", cfg_thresh,   "100");
	bindParameter("3D_Translate X", cfg_translate_x,   "0");
	bindParameter("3D_Translate Y", cfg_translate_y,   "0");
	bindParameter("3D_Translate Z", cfg_translate_z,   "0");
	bindParameter("3D_Rotate ANGLE", cfg_rotate_angle,   "0");
	bindParameter("3D_Rotate X", cfg_rotate_x,   "0");
	bindParameter("3D_Rotate Y", cfg_rotate_y,   "0");
	bindParameter("3D_Rotate Z", cfg_rotate_z,   "0");
	bindParameter("3D_Scale ALL", cfg_scale_all,   "1.0");
	bindParameter("3D_Scale X", cfg_scale_x,   "1.0");
	bindParameter("3D_Scale Y", cfg_scale_y,   "1.0");
	bindParameter("3D_Scale Z", cfg_scale_z,   "1.0");

	bindParameter("HRP3_RLEG Front Back", cfg_rotate_angle_RLEG,   "0");

	arDebug = 1;

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t art::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t art::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t art::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t art::onActivated(RTC::UniqueId ec_id)
{
	m_in_height = 0;
	m_in_width = 0;

	ar_image = NULL;
	opencv_image									= NULL;//cv`p
	opencv_marker_image						= NULL;//cvop
	opencv_img_send_tmp						= NULL;//cvop
	opencv_img_send_mono_tmp			= NULL;
	opencv_img_send								= NULL;//cv`p
	opencv_img_send_mono					= NULL;

	if( 0 == strcmp(patt_name.c_str(), cfg_patt_name.c_str()) )
	{
		patt_id = arLoadPatt(patt_name.c_str());
	}

	if( 0 == strcmp(vrml_name.c_str(), cfg_vrml_name.c_str()) )
	{
		if(vrml_name == "HRP-3 JOINT CONTROL")
		{
			vrml_id_hrp[0] = arVrmlLoadFile("vrml/hrp-3/HRP3_WAIST.dat");
			vrml_id_hrp[1] = arVrmlLoadFile("vrml/hrp-3/HRP3_RLEG.dat"); 
		}
		else
		{
			vrml_id = arVrmlLoadFile(vrml_name.c_str());
		}
	}

	ARMainLoopStopFlag = FALSE;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t art::onDeactivated(RTC::UniqueId ec_id)
{
	ARMainLoopStopFlag = TRUE;
	arUtilSleep(500);

	ar_image = NULL;
	cvReleaseImage(&opencv_marker_image);
	cvReleaseImage(&opencv_image);
	cvReleaseImage(&opencv_img_send);
	cvReleaseImage(&opencv_img_send_mono);
	cvReleaseImage(&opencv_img_send_tmp);
	cvReleaseImage(&opencv_img_send_mono_tmp);

	arFreePatt(patt_id);
	arVrmlFree(vrml_id);
	arVrmlFree(vrml_id_hrp[0]);
	arVrmlFree(vrml_id_hrp[1]);

  return RTC::RTC_OK;
}


RTC::ReturnCode_t art::onExecute(RTC::UniqueId ec_id)
{
	cfgCheckCparamName();
	cfgCheckPattName();
	cfgCheckVrmlName();
	cfgCheckThreshAnd3dCtr();
	if(vrml_name == "HRP-3 JOINT CONTROL")
		cfgCheckHRPCtr();

	if (!m_inIn.isNew())
	{	
		return RTC::RTC_OK;
	}
	m_inIn.read();

	if (!(m_in.pixels.length() > 0))
	{
		return RTC::RTC_OK;
	}

	if(m_in_height != m_in.height || m_in_width != m_in.width || opencv_image == NULL)
	{
		ARMainLoopStopFlag = TRUE;
		arUtilSleep(500);

		m_in_height = m_in.height;
		m_in_width = m_in.width;

		if(opencv_image != NULL)
			cvReleaseImage(&opencv_image);
		if(opencv_marker_image != NULL)
			cvReleaseImage(&opencv_marker_image);
		if(opencv_img_send_tmp != NULL)
			cvReleaseImage(&opencv_img_send_tmp);
		if(opencv_img_send_mono_tmp != NULL)
			cvReleaseImage(&opencv_img_send_mono_tmp);
		if(opencv_img_send != NULL)
			cvReleaseImage(&opencv_img_send);
		if(opencv_img_send_mono != NULL)
			cvReleaseImage(&opencv_img_send_mono);

		opencv_image									= cvCreateImage(cvSize(m_in.width, m_in.height), IPL_DEPTH_8U, 3);//cv`p
		opencv_marker_image						= cvCreateImage(cvSize(m_in.width, m_in.height), IPL_DEPTH_8U, 4);//cvop
		opencv_img_send_tmp						= cvCreateImage(cvSize(m_in.width, m_in.height), IPL_DEPTH_8U, 3);//cvop
		opencv_img_send_mono_tmp = cvCreateImage(cvSize(m_in.width, m_in.height), IPL_DEPTH_8U, 3);
		opencv_img_send								= cvCreateImage(cvSize(m_in.width, m_in.height), IPL_DEPTH_8U, 3);//cv`p
		opencv_img_send_mono			= cvCreateImage(cvSize(m_in.width, m_in.height), IPL_DEPTH_8U, 3);

		ARMainLoopStopFlag = FALSE;
	}

	// m_in.pixels[0] -> opencv_image (in_port -> cv`p)
	memcpy(opencv_image -> imageData, (void *)&(m_in.pixels[0]), m_in.pixels.length());

	// opencv_image -> opencv_marker_image (cv`p -> cvop)(24bpp32bpp֕ϊ))
	cvCvtColor(opencv_image, opencv_marker_image, CV_BGR2BGRA);

	// opencv_marker_image -> ar_image (cvop -> arop)
	ar_image = (ARUint8*)(opencv_marker_image -> imageData);
	
	if(opencv_img_send_tmp -> imageData != NULL || opencv_img_send_mono_tmp != NULL)
	{
		// opencv_img_send_tmp(cvop) -> opencv_img_send(cv`p) 
		cvCvtColor(opencv_img_send_tmp, opencv_img_send, CV_RGB2BGR);
		cvCvtColor(opencv_img_send_mono_tmp, opencv_img_send_mono, CV_RGB2BGR);

		cvFlip(opencv_img_send, opencv_img_send);
		cvFlip(opencv_img_send_mono, opencv_img_send_mono);

		// ʂ̃TCY
		int len = opencv_img_send->nChannels * opencv_img_send->width * opencv_img_send->height;
		int len_mono = opencv_img_send_mono->nChannels * opencv_img_send_mono->width 
			* opencv_img_send_mono->height;

		m_out.pixels.length(len);
		m_out.width  = opencv_img_send->width;
		m_out.height = opencv_img_send->height;

		m_out_mono.pixels.length(len_mono);
		m_out_mono.width  = opencv_img_send_mono->width;
		m_out_mono.height = opencv_img_send_mono->height;

		// opencv_img_send(cv`p) -> m_out.pixels[0]
		memcpy((void *)&(m_out.pixels[0]), opencv_img_send -> imageData, len);
		memcpy((void *)&(m_out_mono.pixels[0]), opencv_img_send_mono -> imageData, len_mono);

		m_outOut.write();
		m_out_monoOut.write();
	}

	static coil::TimeValue tm_pre;
	static int count = 0;

	if (count > 100)
	{
		count = 0;
		coil::TimeValue tm;
		tm = coil::gettimeofday();
		double sec(tm - tm_pre);

		if (sec > 1.0 && sec < 1000.0)
		{
			std::cout << 100.0/sec << " [FPS]" << std::endl;
		}
		tm_pre = tm;
	}
	++count;

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t art::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t art::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t art::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t art::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t art::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

// ============================================================================
//	Functions
// ============================================================================

// cparam_name Configuration Parameter changed
bool art::cfgCheckCparamName(void)
{
	if( 0 != strcmp(cparam_name.c_str(), cfg_cparam_name.c_str()) ) 
	{
		ARMainLoopStopFlag = TRUE;
		arUtilSleep(100);

		cparam_name = cfg_cparam_name;

		if ( arParamLoad( cparam_name.c_str(), 1, &wparam ) < 0 ) 
		{
			if(paramInitFlag == FALSE)
			{
				return RTC::RTC_ERROR;
			}
		}

		arParamChangeSize( &wparam, wparam.xsize, wparam.ysize, &cparam );
		arInitCparam( &cparam );

		arParamDisp( &cparam );

		ARMainLoopStopFlag = FALSE;
		paramInitFlag = TRUE;

		return true;
	}
	return false;
}

// patt_name Configuration Parameter changed
bool art::cfgCheckPattName(void)
{	
	if( 0 != strcmp(patt_name.c_str(), cfg_patt_name.c_str()) )
	{
		ARMainLoopStopFlag = TRUE;
		arUtilSleep(100);

		patt_name = cfg_patt_name;

		arFreePatt(patt_id);
		patt_id = arLoadPatt(patt_name.c_str());

		ARMainLoopStopFlag = FALSE;

		return true;
	}
	return false;
}

// vrml_name Configuration Parameter changed
bool art::cfgCheckVrmlName(void)
{
	if( 0 != strcmp(vrml_name.c_str(), cfg_vrml_name.c_str()) )
	{
		ARMainLoopStopFlag = TRUE;
		arUtilSleep(100);

		vrml_name = cfg_vrml_name;

		if(vrml_name == "HRP-3 JOINT CONTROL")
		{
			arVrmlFree(vrml_id);
			vrml_id_hrp[0] = arVrmlLoadFile("vrml/hrp-3/HRP3_WAIST.dat");
			vrml_id_hrp[1] = arVrmlLoadFile("vrml/hrp-3/HRP3_RLEG.dat"); 
		}

		else
		{
			arVrmlFree(vrml_id_hrp[0]);
			arVrmlFree(vrml_id_hrp[1]);
			arVrmlFree(vrml_id);
			vrml_id = arVrmlLoadFile(vrml_name.c_str()); 
		}

		ARMainLoopStopFlag = FALSE;

		return true;
	}
	return false;
}

// thresh Configuration Parameter changed
bool art::cfgCheckThreshAnd3dCtr(void)
{
	if( thresh != cfg_thresh || translate_x != cfg_translate_x 
		|| translate_y != cfg_translate_y || translate_z != cfg_translate_z 
		|| rotate_angle != cfg_rotate_angle || rotate_x != cfg_rotate_x 
		|| rotate_y != cfg_rotate_y || rotate_z != cfg_rotate_z || scale_all != cfg_scale_all
		|| scale_x != cfg_scale_x || scale_y != cfg_scale_y || scale_z != cfg_scale_z) 
	{
		thresh = cfg_thresh;

		translate_x = cfg_translate_x;
		translate_y = cfg_translate_y;
		translate_z = cfg_translate_z;
		rotate_angle = cfg_rotate_angle;
		rotate_x = cfg_rotate_x;
		rotate_y = cfg_rotate_y;
		rotate_z = cfg_rotate_z;
		
		if(scale_all != cfg_scale_all)
		{
			scale_all = cfg_scale_all;
			scale_x_all = cfg_scale_all;
			scale_y_all = cfg_scale_all;
			scale_z_all = cfg_scale_all;
			scale_all_flag = TRUE;
		}

		if(scale_x != cfg_scale_x || scale_y != cfg_scale_y || scale_z != cfg_scale_z)
		{
			scale_x = cfg_scale_x;
			scale_y = cfg_scale_y;
			scale_z = cfg_scale_z;
			scale_all_flag = FALSE;
		}
		return true;
	}
	return false;
}

// HRP CONTROL Configuration Parameter changed
bool art::cfgCheckHRPCtr(void)
{
	if( rotate_angle_RLEG != cfg_rotate_angle_RLEG ) 
	{
		rotate_angle_RLEG = cfg_rotate_angle_RLEG;

		return true;
	}

	return false;
}


void activeWaitLoop(void)
{
	for(;;)
	{
		if(paramInitFlag == TRUE)
		{
			argInit( &cparam, 1.0, 0, 0, 0, 0 );

			glewInit();
			InitTexture();
			InitRenderbuffer();
			InitFramebuffer();

			glutDisplayFunc( Display );
			glutIdleFunc(AR_MainLoop);

			glutMainLoop();
		}
	}
}


//===========================================================================
// C[v֐
//===========================================================================
void Display(void)
{
	glutHideWindow();
	AR_MainLoop();
}

void AR_MainLoop(void)
{
	if(ARMainLoopStopFlag == TRUE)
	{
		return;
	}

	if ( ar_image == NULL ) 
	{
		arUtilSleep(2);
		return;
	}

	static int		isFirst = 1;	// tO
	ARMarkerInfo	*marker_info;
	int				marker_num;
	int				j, k, i;

	// Framebuffer̐ݒ
	glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, framebuffer_name );

	// clear buffer
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	// VRMLp^C}̍XV
	arVrmlTimerUpdate();

	// ---- 1) ar_image(2D camera image) -> buffer  ----
	// J摜̕`
	argDrawMode2D();
	argDispImage( ar_image, 0, 0 );
	

	// ---- 2) ar_image(2D camera image) -> p^[F -> 3D object -> buffer ----
	// }[J̌oƔF
	if ( arDetectMarker( ar_image, thresh, &marker_info, &marker_num ) < 0 ) 
	{
		return;
	}

	// }[J̐Mx̔r
	k = -1;
	for( j = 0; j < marker_num; j++ ) 
	{
		if ( patt_id == marker_info[j].id ) 
		{
			if ( k == -1 )
			{
				k = j;
			}
			else if ( marker_info[k].cf < marker_info[j].cf ) 
			{
				k = j;
			}
		}
	}

	if ( k != -1 ) 
	{
		// }[J̈ʒuEpiWϊsǰvZ
		if ( isFirst ) 
		{
			arGetTransMat( &marker_info[k], patt_center, patt_width, patt_trans );
		} 
		else 
		{
			// CG̕\艻
			arGetTransMatCont( &marker_info[k], patt_trans, patt_center, patt_width, patt_trans );
		}
		isFirst = 0;	// tO
	
		// 3DIuWFNg̕`
		DrawObject();
	}

	glReadPixels(0, 0, m_in_width, m_in_height, GL_RGB, GL_UNSIGNED_BYTE, 
		opencv_img_send_tmp->imageData);
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );


	// OutPort2_mono_debug_mode
	argDrawMode2D();
	argDispImage( arImage, 0, 0);

	if ( k != -1 ) 
	{
		glColor3f( 1.0, 0.0, 0.0 );
		glLineWidth( 3.0 );
		for( i = 0; i < marker_num; i++ ) 
		{
			if( marker_info[i].id < 0 ) 
				continue;
			argDrawSquare( marker_info[i].vertex, 0, 0 );
		}
		glLineWidth( 1.0 );
	}

	glReadPixels(0, 0, m_in_width, m_in_height, GL_RGB, GL_UNSIGNED_BYTE, 
		opencv_img_send_mono_tmp->imageData);
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
}

//===========================================================================
// 3DIuWFNg̕`s֐
//===========================================================================
void DrawObject(void)
{
	double	gl_para[16];

	// 3DIuWFNg`悷邽߂̏
	argDrawMode3D();
	argDraw3dCamera( 0, 0 );

	// Wϊs̓Kp
	argConvGlpara( patt_trans, gl_para );
	glMatrixMode( GL_MODELVIEW );
	glLoadMatrixd( gl_para );

	// 3DIuWFNg̕`
	glClear( GL_DEPTH_BUFFER_BIT );		// Zobt@̏
	glEnable( GL_DEPTH_TEST );			// Bʏ̓Kp

	if(vrml_name == "HRP-3 JOINT CONTROL")
	{
		arVrmlDraw(vrml_id_hrp[0]);

		glTranslatef (0, -20, 180); 
		glRotatef (rotate_angle_RLEG, 0, -1, 0);
		arVrmlDraw(vrml_id_hrp[1]);
	}

	else
	{
		glTranslatef (translate_x, translate_y, translate_z); 
		glRotatef (rotate_angle, rotate_x, rotate_y, rotate_z); 

		if(scale_all_flag == FALSE)
		{
			glScalef (scale_x, scale_y, scale_z);
		}

		else
		{
			glScalef (scale_x_all, scale_y_all, scale_z_all);
		}

		arVrmlDraw(vrml_id);				// VRML̕`
	}

	glDisable( GL_DEPTH_TEST );
}

//===========================================================================
// Frame Buffer Object̂߂glew֐
//===========================================================================

void InitTexture( void )
{
	glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
	glGenTextures( 1, &texture_name );
	glBindTexture( GL_TEXTURE_2D, texture_name );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB, TEXTURE_WIDTH, TEXTURE_HEIGHT,	
		0, GL_RGB, GL_UNSIGNED_BYTE, 0 );
}

void InitRenderbuffer( void )
{
	glGenRenderbuffersEXT( 1, &renderbuffer_name );
	glBindRenderbufferEXT( GL_RENDERBUFFER_EXT, renderbuffer_name );
	glRenderbufferStorageEXT( GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT,
		RENDERBUFFER_WIDTH, RENDERBUFFER_HEIGHT );
}

void InitFramebuffer( void )
{
	glGenFramebuffersEXT( 1, &framebuffer_name );
	glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, framebuffer_name );

	// Texture Object  FBO (attach a texture to FBO color attachement point)
	glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, texture_name, 0 );

	// RBO  FBO (attach a renderbuffer to depth attachment point)
	glFramebufferRenderbufferEXT( GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT,
		GL_RENDERBUFFER_EXT, renderbuffer_name );

	glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );
}

extern "C"
{
 
  void artInit(RTC::Manager* manager)
  {
    coil::Properties profile(art_spec);
    manager->registerFactory(profile,
                             RTC::Create<art>,
                             RTC::Delete<art>);
  }
  
};
