// -*- C++ -*-
/*!
 * @file  art.h
 * @brief ARToolkit image capture component
 * @date  $Date$
 *
 * $Id$
 */

#ifndef ART_H
#define ART_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// For CameraImage port
#include <rtm/idl/InterfaceDataTypesSkel.h>


// ============================================================================
//	Includes
// ============================================================================
// OpenCV
#include <cv.h>
#include <cxcore.h>
#include <highgui.h>

// Windows
#include <windows.h>
#include <stdio.h>


// OpenGL
#include <GL/glew.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

// ARToolkit
#include <AR/ar.h>
#include <AR/param.h>
#include <AR/video.h>
#include <AR/gsub.h>
#include <AR/arvrml.h>

// vg^Cv錾
void activeWaitLoop(void);
void Display(void);
void AR_MainLoop(void);
void DrawObject(void);

/* framebuffer object */

void InitTexture( void );
void InitRenderbuffer( void );
void InitFramebuffer( void );

#define	FRAMEBUFFER_WIDTH		640
#define	FRAMEBUFFER_HEIGHT		480

#define	TEXTURE_WIDTH			FRAMEBUFFER_WIDTH
#define	TEXTURE_HEIGHT			FRAMEBUFFER_HEIGHT

#define	RENDERBUFFER_WIDTH		FRAMEBUFFER_WIDTH
#define	RENDERBUFFER_HEIGHT		FRAMEBUFFER_HEIGHT


// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

using namespace RTC;

/*!
 * @class art
 * @brief ARToolkit image capture component
 *
 */
class art
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  art(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~art();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /**
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">

  CameraImage m_in;
  InPort<CameraImage> m_inIn;

  CameraImage m_out;
  OutPort<CameraImage> m_outOut;
  
	CameraImage m_out_mono;
  OutPort<CameraImage> m_out_monoOut;

  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:

	 // Image Variable
	 int dummy;
	 IplImage* opencv_image;
	 IplImage* opencv_marker_image;
	 IplImage* opencv_img_send;
	 IplImage* opencv_img_send_mono;

	 // ConfigurationSet Variable
	 std::string cfg_cparam_name;
	 std::string cfg_patt_name;
	 std::string cfg_vrml_name;
	 int cfg_thresh;

	 GLfloat cfg_translate_x;
	 GLfloat cfg_translate_y;
	 GLfloat cfg_translate_z;
	 GLfloat cfg_rotate_angle;
	 GLfloat cfg_rotate_x;
	 GLfloat cfg_rotate_y;
	 GLfloat cfg_rotate_z;
	 GLfloat cfg_scale_all;
	 GLfloat cfg_scale_x;
	 GLfloat cfg_scale_y;
	 GLfloat cfg_scale_z;

	 GLfloat cfg_rotate_angle_RLEG;


	 // ConfigurationSet Fuction
	 bool cfgCheckCparamName(void);
	 bool cfgCheckPattName(void);
	 bool cfgCheckVrmlName(void);
	 bool cfgCheckThreshAnd3dCtr(void);
	 bool cfgCheckHRPCtr(void);

  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>
};


extern "C"
{
  void artInit(RTC::Manager* manager);
};

#endif // ART_H
